#!/usr/bin/env python3

"""
Nemesida WAF API
Copyright (c) 2017-2025 Pentestit LLC - All Rights Reserved
Author: Romanov R.
"""

import base64
import json
import multiprocessing as mp

from ast import literal_eval
from fastapi import FastAPI, Request
from fastapi.responses import Response, JSONResponse

from urls import del_agent_monitoring_v1_uri
from urls import del_antibot_v1_uri
from urls import del_attack_uri
from urls import del_dyn_sync_ban_uri
from urls import del_event_uri
from urls import del_mls_data_v1_uri
from urls import del_models_uri
from urls import del_models_v2_uri
from urls import del_mt_attack_uri
from urls import del_mt_fp_uri
from urls import del_nws_target_uri
from urls import get_agent_monitoring_v1_uri
from urls import get_antibot_summary_v1_uri
from urls import get_antibot_v1_uri
from urls import get_attack_spec_uri
from urls import get_attack_stats_uri
from urls import get_attack_uri
from urls import get_attack_vhost_stats_uri
from urls import get_dyn_bl_hash_uri
from urls import get_dyn_bl_uri
from urls import get_dyn_bl_v2_uri
from urls import get_dyn_erl_hash_uri
from urls import get_dyn_erl_uri
from urls import get_dyn_erl_v2_uri
from urls import get_dyn_settings_hash_uri
from urls import get_dyn_settings_uri
from urls import get_dyn_settings_v2_uri
from urls import get_dyn_sync_ban_uri
from urls import get_dyn_wl_hash_uri
from urls import get_dyn_wl_uri
from urls import get_dyn_wl_v2_uri
from urls import get_event_uri
from urls import get_mlc_settings_hash_uri
from urls import get_mlc_settings_uri
from urls import get_mlc_settings_v2_uri
from urls import get_mls_data_v1_uri
from urls import get_models_list_uri
from urls import get_models_list_v2_uri
from urls import get_models_uri
from urls import get_mt_attack_uri
from urls import get_mt_data_uri
from urls import get_mt_fp_uri
from urls import get_mt_hash_uri
from urls import get_nginx_certificate_hash_uri
from urls import get_nginx_certificate_secure_uri
from urls import get_nginx_certificate_uri
from urls import get_nginx_dhparam_hash_uri
from urls import get_nginx_dhparam_secure_uri
from urls import get_nginx_dhparam_uri
from urls import get_nginx_global_hash_uri
from urls import get_nginx_global_uri
from urls import get_nginx_upstream_hash_uri
from urls import get_nginx_upstream_uri
from urls import get_nginx_vhost_hash_uri
from urls import get_nginx_vhost_uri
from urls import get_nws_target_uri
from urls import get_openapi_exclude_paths_hash_uri
from urls import get_openapi_exclude_paths_uri
from urls import get_openapi_schema_paths_uri
from urls import get_openapi_schemas_hash_uri
from urls import get_openapi_schemas_uri
from urls import get_openapi_url_hash_uri
from urls import get_openapi_url_uri
from urls import get_training_uri
from urls import get_training_v2_uri
from urls import get_vhosts_list_uri
from urls import get_vhosts_list_v2_uri
from urls import rep_models_uri
from urls import rep_models_v2_uri
from urls import set_agent_monitoring_v1_uri
from urls import set_antibot_v1_uri
from urls import set_attack_uri
from urls import set_dyn_bl_uri
from urls import set_dyn_bl_v2_uri
from urls import set_dyn_erl_uri
from urls import set_dyn_erl_v2_uri
from urls import set_dyn_settings_uri
from urls import set_dyn_settings_v2_uri
from urls import set_dyn_sync_ban_uri
from urls import set_dyn_wl_uri
from urls import set_dyn_wl_v2_uri
from urls import set_event_uri
from urls import set_mlc_settings_uri
from urls import set_mlc_settings_v2_uri
from urls import set_mls_data_v1_uri
from urls import set_models_uri
from urls import set_mt_data_uri
from urls import set_mt_fp_uri
from urls import set_nginx_certificate_uri
from urls import set_nginx_dhparam_uri
from urls import set_nginx_global_uri
from urls import set_nginx_upstream_uri
from urls import set_nginx_vhost_uri
from urls import set_nws_target_uri
from urls import set_openapi_exclude_paths_uri
from urls import set_openapi_schema_paths_uri
from urls import set_openapi_schema_uri
from urls import set_openapi_url_uri
from urls import set_scan_uri
from urls import set_training_progress_uri
from urls import set_training_uri
from urls import set_training_v2_uri
from urls import set_vhosts_list_uri
from urls import set_vhosts_list_v2_uri

from core import geo_token_update
from core import get_agent_ip
from core import get_auth
from core import get_waf_id
from core import memc
from core import memc_lock
from core import request_form_preprocess
from core import request_json_preprocess
from core import ro_mode

from asr import del_attack
from asr import get_attack
from asr import get_attack_spec
from asr import get_attack_stats
from asr import get_attack_vhost_stats
from asr import set_attack
from asr import set_scan

from ws import del_nws_target
from ws import get_dyn_bl
from ws import get_dyn_bl_v2
from ws import get_dyn_erl
from ws import get_dyn_erl_v2
from ws import get_dyn_settings
from ws import get_dyn_settings_v2
from ws import get_dyn_wl
from ws import get_dyn_wl_v2
from ws import get_hash
from ws import get_mlc_settings
from ws import get_mlc_settings_v2
from ws import get_nws_target
from ws import set_dyn_bl
from ws import set_dyn_bl_v2
from ws import set_dyn_erl
from ws import set_dyn_erl_v2
from ws import set_dyn_settings
from ws import set_dyn_settings_v2
from ws import set_dyn_wl
from ws import set_dyn_wl_v2
from ws import set_mlc_settings
from ws import set_mlc_settings_v2
from ws import set_nws_target

from bs import del_dyn_sync_ban
from bs import get_dyn_sync_ban
from bs import set_dyn_sync_ban

from mt import del_mt_attack
from mt import del_mt_fp
from mt import get_mt_attack
from mt import get_mt_fp
from mt import mt_get_export
from mt import mt_get_export_md5
from mt import set_mt_attack
from mt import set_mt_fp

from ml import del_models
from ml import get_models
from ml import get_models_list
from ml import get_training
from ml import get_training_v2
from ml import get_vhosts_list
from ml import rep_models
from ml import set_models
from ml import set_training
from ml import set_training_progress
from ml import set_training_v2
from ml import set_vhosts_list

from oa import get_openapi_exclude_paths
from oa import get_openapi_schema
from oa import get_openapi_schema_paths
from oa import get_openapi_url
from oa import set_openapi_exclude_paths
from oa import set_openapi_schema
from oa import set_openapi_schema_paths
from oa import set_openapi_url

from ns import get_nginx
from ns import get_nginx_hash
from ns import nginx_certificate_cols
from ns import nginx_dhparam_cols
from ns import nginx_global_cols
from ns import nginx_upstream_cols
from ns import nginx_vhost_cols
from ns import set_nginx

from ec import del_event
from ec import get_event
from ec import set_event

from am import del_agent_monitoring_v1
from am import get_agent_monitoring_v1
from am import set_agent_monitoring_v1

from mls import del_mls_data_v1
from mls import get_mls_data_v1
from mls import set_mls_data_v1

from ab import del_antibot_v1
from ab import get_antibot_summary_v1
from ab import get_antibot_v1
from ab import set_antibot_v1

from logger import log


##
# Launch the API
##

app = FastAPI(redirect_slashes=False, ocs_url=None, redoc_url=None, openapi_url=None)
log.info('START')


##
# Attacks result
##


@app.post(get_attack_uri)
@app.post(get_attack_stats_uri)
@app.post(get_attack_vhost_stats_uri)
@app.post(get_attack_spec_uri)
@app.post(del_attack_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_attack_uri, get_attack_stats_uri, get_attack_vhost_stats_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, _, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # remove the key
        del data['key']

        # data processing
        if url == get_attack_uri:
            r = get_attack(url, data, agent_ip, waf_id)
        elif url == get_attack_stats_uri:
            r = get_attack_stats(data, waf_id)
        elif url == get_attack_vhost_stats_uri:
            r = get_attack_vhost_stats(data, waf_id)
        elif url == get_attack_spec_uri:
            r = get_attack_spec(waf_id)
        elif url == del_attack_uri:
            r = del_attack(data, waf_id)
        else:
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_attack_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # extract client IP
        agent_ip = get_agent_ip(request)

        # extract the data
        try:
            data = json.loads(await request.body())
        except Exception as e:
            log.error('An error occurred while converting data JSON in {}: {} ({})'.format(url, e, await request.body()))
            return

        # save attack in the database
        p = mp.Process(target=set_attack, args=(data, agent_ip,))
        p.daemon = False
        p.start()

        # response
        return Response(status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Scanning result
##

@app.post(set_scan_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # extract client IP
        agent_ip = get_agent_ip(request)

        try:
            data = json.loads(await request.body())
        except Exception as e:
            log.error('An error occurred while converting data JSON in {}: {} ({})'.format(url, e, await request.body()))
            return Response(status_code=400)

        # response
        r = set_scan(url, rsc, data, agent_ip)
        return Response(status_code=r) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# DynSyncBan
##

@app.post(get_dyn_sync_ban_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_sync_ban(url, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_sync_ban_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_sync_ban(url, data, key, agent_ip, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(del_dyn_sync_ban_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = del_dyn_sync_ban(url, data, key, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# OpenAPI
##

@app.post(get_openapi_url_uri)
@app.post(set_openapi_url_uri)
@app.post(set_openapi_schema_uri)
@app.post(get_openapi_schemas_uri)
@app.post(get_openapi_schema_paths_uri)
@app.post(get_openapi_exclude_paths_uri)
@app.post(set_openapi_exclude_paths_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_openapi_url_uri, get_openapi_schemas_uri, get_openapi_schema_paths_uri, get_openapi_exclude_paths_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # data processing
        if url == get_openapi_url_uri:
            r = get_openapi_url(waf_id)
        elif url == set_openapi_url_uri:
            r = set_openapi_url(data, waf_id)
        elif url == set_openapi_schema_uri:
            r = set_openapi_schema(data, agent_ip, waf_id)
        elif url == get_openapi_schemas_uri:
            r = get_openapi_schema(waf_id)
        elif url == get_openapi_schema_paths_uri:
            r = get_openapi_schema_paths(waf_id)
        elif url == get_openapi_exclude_paths_uri:
            r = get_openapi_exclude_paths(waf_id)
        elif url == set_openapi_exclude_paths_uri:
            r = set_openapi_exclude_paths(data, waf_id)
        else:
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_openapi_schema_paths_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # extract client IP
        agent_ip = get_agent_ip(request)

        try:
            data = json.loads(await request.body())
        except Exception as e:
            log.error('An error occurred while converting data JSON in {}: {} ({})'.format(url, e, await request.body()))
            return Response(status_code=400)

        # extract the action
        set_path = data.get('set', {})
        del_path = data.get('del', {})

        # extract lic. key
        if 'key' in data:
            # default key location
            key = data.get('key')
        else:
            # previous key location
            key = set_path.get('key') if set_path else del_path.get('key')

        # WAF ID check
        waf_id = get_waf_id(key, agent_ip)
        if not waf_id:
            return Response(status_code=401)

        # response
        r = set_openapi_schema_paths(set_path, del_path, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_openapi_exclude_paths_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # get data from the memcached
        memc_key = 'hash_exclude_path_' + waf_id
        with memc_lock:
            try:
                memc_bin = memc.get(memc_key)
            except Exception as e:
                memc_bin = None
                log.error('An error occurred while receiving data from the memcached: {}'.format(e))

        # processing data
        if memc_bin:
            response = {'status': 'success', 'hash': memc_bin.decode('UTF-8')}
        else:
            response = {'status': 'none'}

        # response
        return Response(str(response), status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_openapi_schemas_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # get data from the memcached
        memc_key = 'hash_schemas_' + waf_id
        with memc_lock:
            try:
                memc_bin = memc.get(memc_key)
            except Exception as e:
                memc_bin = None
                log.error('An error occurred while receiving data from the memcached: {}'.format(e))

        # processing data
        if memc_bin:
            response = {'status': 'success', 'hash': memc_bin.decode('UTF-8')}
        else:
            response = {'status': 'none'}

        # response
        return Response(str(response), status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_openapi_url_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # get data from the memcached
        memc_key = 'hash_url_' + waf_id
        with memc_lock:
            try:
                memc_bin = memc.get(memc_key)
            except Exception as e:
                memc_bin = None
                log.error('An error occurred while receiving data from the memcached: {}'.format(e))

        # processing data
        if memc_bin:
            response = {'status': 'success', 'hash': memc_bin.decode('UTF-8')}
        else:
            response = {'status': 'none'}

        # response
        return Response(str(response), status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Models tuning
##

@app.post(set_mt_data_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # init
        rsc = 201

        # extract client IP
        agent_ip = get_agent_ip(request)

        try:
            data = base64.b64decode(await request.body()).decode('UTF-8', 'ignore')
        except Exception as e:
            log.error('An error occurred while processing the data in {}: {}'.format(url, e))
            return Response(status_code=rsc)

        try:
            data = json.loads(data)
        except Exception as e:
            log.error('An error occurred while converting data JSON in {}: {} ({})'.format(url, e, data))
            return Response(status_code=rsc)

        # extract lic. key
        key = request.cookies.get('key')
        if not key:
            log.error('An error occurred while processing data in {} from client {}: no product key'.format(url, agent_ip))
            return Response(status_code=401)

        # WAF ID check
        waf_id = get_waf_id(key, agent_ip)
        if not waf_id:
            return Response(status_code=401)

        # response
        r = set_mt_attack(url, request, data, agent_ip, waf_id)
        return Response(status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.get(get_mt_data_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # extract client IP
        agent_ip = get_agent_ip(request)

        # extract lic. key
        key = request.cookies.get('key')
        if not key:
            log.error('An error occurred while processing data in {} from client {}: no product key'.format(url, agent_ip))
            return Response(status_code=401)

        # WAF ID check
        waf_id = get_waf_id(key, agent_ip)
        if not waf_id:
            return Response(status_code=401)

        res = mt_get_export(waf_id)
        if res:
            d = json.dumps(res)
            response = base64.b64encode(bytes(d, 'UTF-8'))
            return Response(response, status_code=rsc)
        else:
            return Response(status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.get(get_mt_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # extract client IP
        agent_ip = get_agent_ip(request)

        # extract lic. key
        key = request.cookies.get('key')
        if not key:
            log.error('An error occurred while processing data in {} from client {}: no product key'.format(url, agent_ip))
            return Response(status_code=401)

        # WAF ID check
        waf_id = get_waf_id(key, agent_ip)
        if not waf_id:
            return Response(status_code=401)

        r = mt_get_export_md5(waf_id)
        return Response(r, status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Models tuning: FP MGMT
##

@app.post(get_mt_attack_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = get_mt_attack(url, data, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(del_mt_attack_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = del_mt_attack(data, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_mt_fp_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # remove the key
        del data['key']

        # response
        r = get_mt_fp(url, data, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_mt_fp_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_mt_fp(url, data, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(del_mt_fp_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = del_mt_fp(data, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Models management: v1
##

@app.post(get_models_uri)
async def main(request: Request):

    """
    Status code:
    - 202 : the model exists
    - 203 : the model does not exist
    """

    # set URL
    url = request.scope['path']

    try:

        # extract client IP
        agent_ip = get_agent_ip(request)

        # extract key
        key = request.cookies.get('key')
        if not key:
            log.error('An error occurred while processing data in {} from client {}: no product key'.format(url, agent_ip))
            return Response(status_code=401)

        # WAF ID check
        waf_id = get_waf_id(key, agent_ip)
        if not waf_id:
            return Response(status_code=401)

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # processing data from the request
        data = base64.b64decode(await request.body()).decode('UTF-8')

        # check data from the request
        if not data:
            return Response(status_code=401)

        # response
        r = get_models(data, waf_id)
        return Response(r[1], status_code=r[0]) if r[1] else Response(status_code=r[0])

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_models_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # extract client IP
        agent_ip = get_agent_ip(request)

        # extract key
        key = request.cookies.get('key')
        if not key:
            log.error('An error occurred while processing data in {} from client {}: no product key'.format(url, agent_ip))
            return Response(status_code=401)

        # lic. key validation
        waf_id, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not waf_id) or (not tarif_id) or (not models_num):
            return Response(status_code=401)

        try:

            # extract data
            data = json.loads(await request.body())
            vhost = data['vhost']
            shash = data['hash']
            model = base64.b64decode(data['model'])
            del data

        except Exception as e:
            log.error('An error occurred in {} while processing data of the request: {}'.format(url, e))
            return Response(status_code=400)

        # response
        r = set_models(url, vhost, shash, model, agent_ip, waf_id)
        return Response(status_code=r)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(del_models_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract vhost
        vhost = dict(await request.form()).get('vhost')
        if vhost is None:
            log.error('An error occurred in {} from {}: no vhost'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = del_models(vhost, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(rep_models_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    """
    The wld-vhost for dst-model must be normalized to all, sub_example.com or .example.com.
    The files of wld-vhost for dst-model must be normalized to all, sub_example.com or example.com.
    """

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract the body
        data = dict(await request.form())

        # extract source models vhost
        src = data.get('src')
        if src is None:
            log.error('An error occurred in {} from {}: no src'.format(url, agent_ip))
            return Response(status_code=400)

        # extract dst models vhost
        dst = data.get('dst')
        if dst is None:
            log.error('An error occurred in {} from {}: no dst'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = rep_models(src, dst, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_models_list_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = get_models_list(waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_training_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract vhost
        vhost = dict(await request.form()).get('vhost')
        if vhost is None:
            log.error('An error occurred in {} from {}: no vhost'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = get_training(vhost, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}' .format(url, e))
        return Response(status_code=400)


@app.post(set_training_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract the body
        data = dict(await request.form())

        # extract vhost
        vhost = data.get('vhost')
        if vhost is None:
            log.error('An error occurred in {} from {}: no vhost'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = set_training(url, data, vhost, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_training_progress_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract the body
        body = dict(await request.form())

        # extract vhost
        vhost = body.get('vhost')
        if vhost is None:
            log.error('An error occurred in {} from {}: no vhost'.format(url, agent_ip))
            return Response(status_code=400)

        # extract progress
        progress = body.get('progress')
        if progress is None:
            log.error('An error occurred in {} from {}: no progress'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = set_training_progress(url, progress, vhost, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Models management: v2
##

@app.post(del_models_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract model
        vhost = data.get('model')
        if vhost is None:
            log.error('An error occurred in {} from {}: no model'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = del_models(vhost, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(rep_models_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    """
    The wld-vhost for dst-model must be normalized to all, sub_example.com or .example.com.
    The files of wld-vhost for dst-model must be normalized to all, sub_example.com or example.com.
    """

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract source models vhost
        src = data.get('src_model')
        if src is None:
            log.error('An error occurred in {} from {}: no src'.format(url, agent_ip))
            return Response(status_code=400)

        # extract dst models vhost
        dst = data.get('dst_model')
        if dst is None:
            log.error('An error occurred in {} from {}: no dst'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = rep_models(src, dst, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_models_list_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = {'status': 'success', 'models': get_models_list(waf_id)}
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_training_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = get_training_v2(waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_training_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # extract vhost
        vhost = data.get('vhost')
        if vhost is None:
            log.error('An error occurred in {} from {}: no vhost'.format(url, agent_ip))
            return Response(status_code=400)

        # response
        r = set_training_v2(data, vhost, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# ML Virtual Hosts Management
##

@app.post(get_vhosts_list_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = get_vhosts_list(waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_vhosts_list_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = set_vhosts_list(url, data, models_num, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_vhosts_list_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = get_vhosts_list(waf_id)
        r['vhosts_list'] = literal_eval(r['vhosts_list'])
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_vhosts_list_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # lic. key validation
        _, tarif_id, models_num, _ = get_auth(key, agent_ip)

        # Business/Enterprise only with models
        if (not tarif_id) or (not models_num):
            return Response(status_code=401)

        # response
        r = set_vhosts_list(url, data, models_num, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Agent Settings API: v1
##

@app.post(get_mlc_settings_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # processing request
        t = 'mlc_settings'
        response, code = get_hash(agent_ip, url, t, waf_id)

        # response
        if code == 200:
            return Response(response, status_code=code)
        else:
            return Response(status_code=code)

    except Exception as e:
        log.error('An error occurred in {}'.format(url, e))


@app.post(get_mlc_settings_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_mlc_settings(request, agent_ip, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_mlc_settings_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_mlc_settings(url, data, key, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_settings_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # processing request
        t = 'dyn_settings'
        response, code = get_hash(agent_ip, url, t, waf_id)

        # response
        if code == 200:
            return Response(response, status_code=code)
        else:
            return Response(status_code=code)

    except Exception as e:
        log.error('An error occurred in {}'.format(url, e))


@app.post(get_dyn_settings_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_settings(request, agent_ip, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_settings_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_settings(url, data, key, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_erl_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # processing request
        t = 'dyn_erl_ts'
        response, code = get_hash(agent_ip, url, t, waf_id)

        # response
        if code == 200:
            return Response(response, status_code=code)
        else:
            return Response(status_code=code)

    except Exception as e:
        log.error('An error occurred in {}'.format(url, e))


@app.post(get_dyn_erl_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_erl(request, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_erl_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_erl(url, data, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_wl_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # processing request
        t = 'dyn_wl_ts'
        response, code = get_hash(agent_ip, url, t, waf_id)

        # response
        if code == 200:
            return Response(response, status_code=code)
        else:
            return Response(status_code=code)

    except Exception as e:
        log.error('An error occurred in {}'.format(url, e))


@app.post(get_dyn_wl_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_wl(request, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_wl_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_wl(url, data, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_bl_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # processing request
        t = 'dyn_bl'
        response, code = get_hash(agent_ip, url, t, waf_id)

        # response
        if code == 200:
            return Response(response, status_code=code)
        else:
            return Response(status_code=code)

    except Exception as e:
        log.error('An error occurred in {}'.format(url, e))


@app.post(get_dyn_bl_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_bl(request, agent_ip, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_bl_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return Response(str(r[1]), status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_bl(url, data, agent_ip, waf_id)
        return Response(str(r), status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Agent Settings API: v2
##

@app.post(get_mlc_settings_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_mlc_settings_v2(agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_mlc_settings_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_mlc_settings_v2(url, data, key, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_settings_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_settings_v2(agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_settings_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_settings_v2(url, data, key, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_erl_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_erl_v2(waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_erl_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_erl_v2(url, data, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_wl_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_wl_v2(waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_wl_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_wl_v2(url, data, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_dyn_bl_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # response
        r = get_dyn_bl_v2(agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_dyn_bl_v2_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # response
        r = set_dyn_bl_v2(url, data, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Nginx Settings
##

@app.post(get_nginx_global_hash_uri)
@app.post(get_nginx_vhost_hash_uri)
@app.post(get_nginx_upstream_hash_uri)
@app.post(get_nginx_certificate_hash_uri)
@app.post(get_nginx_dhparam_hash_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200

        # set table
        if url == get_nginx_global_hash_uri:
            table = 'nginx_' + 'global'
        elif url == get_nginx_vhost_hash_uri:
            table = 'nginx_' + 'vhost'
        elif url == get_nginx_upstream_hash_uri:
            table = 'nginx_' + 'upstream'
        elif url == get_nginx_certificate_hash_uri:
            table = 'nginx_' + 'certificate'
        elif url == get_nginx_dhparam_hash_uri:
            table = 'nginx_' + 'dhparam'
        else:
            err_msg = 'An unknown request path {}'.format(url)
            log.error(err_msg)
            response = {'status': 'fail', 'description': err_msg}
            return JSONResponse(response, status_code=rsc)

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # extract the hash
        response = get_nginx_hash(table, waf_id)

        # processing result
        if not response:
            response = {'status': 'none'}
        else:
            response = {'status': 'success', 'hash': response}

        # response
        return JSONResponse(response, status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(get_nginx_global_uri)
@app.post(get_nginx_vhost_uri)
@app.post(get_nginx_upstream_uri)
@app.post(get_nginx_dhparam_uri)
@app.post(get_nginx_certificate_uri)
@app.post(get_nginx_dhparam_secure_uri)
@app.post(get_nginx_certificate_secure_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200
        get_nginx_with_active = [get_nginx_global_uri, get_nginx_vhost_uri]
        get_nginx_with_format = [get_nginx_global_uri, get_nginx_vhost_uri, get_nginx_upstream_uri]

        # body request preprocessing
        r = await request_form_preprocess(url, request)
        if r[0]:
            key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
        else:
            return Response(status_code=r[1])

        # type processing
        if url == get_nginx_global_uri:
            nginx_type = 'global'
        elif url == get_nginx_vhost_uri:
            nginx_type = 'vhost'
        elif url == get_nginx_upstream_uri:
            nginx_type = 'upstream'
        elif url in [get_nginx_dhparam_uri, get_nginx_dhparam_secure_uri]:
            nginx_type = 'dhparam'
        elif url in [get_nginx_certificate_uri, get_nginx_certificate_secure_uri]:
            nginx_type = 'certificate'
        else:
            err_msg = 'An unknown request path {}'.format(url)
            log.error(err_msg)
            response = {'status': 'fail', 'description': err_msg}
            return JSONResponse(response, status_code=rsc)

        # response
        r = get_nginx(url, request, nginx_type, get_nginx_with_active, get_nginx_with_format, waf_id)
        return Response(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


@app.post(set_nginx_global_uri)
@app.post(set_nginx_vhost_uri)
@app.post(set_nginx_upstream_uri)
@app.post(set_nginx_dhparam_uri)
@app.post(set_nginx_certificate_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 201

        # RW DB type only
        if ro_mode:
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # type and columns processing
        if url == set_nginx_global_uri:
            nginx_type = 'global'
            cols = nginx_global_cols
        elif url == set_nginx_vhost_uri:
            nginx_type = 'vhost'
            cols = nginx_vhost_cols
        elif url == set_nginx_upstream_uri:
            nginx_type = 'upstream'
            cols = nginx_upstream_cols
        elif url == set_nginx_certificate_uri:
            nginx_type = 'certificate'
            cols = nginx_certificate_cols
        elif url == set_nginx_dhparam_uri:
            nginx_type = 'dhparam'
            cols = nginx_dhparam_cols
        else:
            err_msg = 'An unknown request path {}'.format(url)
            log.error(err_msg)
            response = {'status': 'fail', 'description': err_msg}
            return JSONResponse(response, status_code=rsc)

        # response
        r = set_nginx(url, data, nginx_type, cols, agent_ip, waf_id)
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Event collector
##

@app.post(get_event_uri)
@app.post(set_event_uri)
@app.post(del_event_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_event_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # data processing
        if url == get_event_uri:
            r = get_event(url, data, agent_ip, waf_id)

        elif url == set_event_uri:

            # extract events
            try:
                events = data['events']
            except Exception as e:
                log.error('An error occurred while extracting the events from {} to {}: {}'.format(agent_ip, url, e))
                return Response(status_code=400)

            # response
            r = set_event(events, waf_id)

        elif url == del_event_uri:
            r = del_event(url, data, agent_ip, waf_id)

        else:
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Scanner settings
##

@app.post(get_nws_target_uri)
@app.post(set_nws_target_uri)
@app.post(del_nws_target_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_nws_target_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # get PlanID
        _, tarif_id, _, _ = get_auth(key, agent_ip)

        # Enterprise only
        if tarif_id < 2:
            return Response(status_code=401)

        # data processing
        if url == get_nws_target_uri:
            r = get_nws_target(waf_id)
        elif url == set_nws_target_uri:
            r = set_nws_target(data, key, agent_ip, waf_id)
        elif url == del_nws_target_uri:
            r = del_nws_target(data, key, waf_id)
        else:
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc) if r is not None else Response(status_code=503)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Agent Monitoring
##

@app.post(get_agent_monitoring_v1_uri)
@app.post(set_agent_monitoring_v1_uri)
@app.post(del_agent_monitoring_v1_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_antibot_v1_uri, get_antibot_summary_v1_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # data processing
        if url == get_agent_monitoring_v1_uri:

            # body request preprocessing
            r = await request_form_preprocess(url, request)
            if r[0]:
                key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2]
            else:
                return Response(status_code=r[1])

            # response
            r = get_agent_monitoring_v1(waf_id)

        elif url == set_agent_monitoring_v1_uri:

            # body request preprocessing
            r = await request_json_preprocess(url, request)
            if r[0]:
                data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
            else:
                return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

            # extract the agent/uuid
            try:
                agent = data['agent']
                ip = data['ip']
                uuid = data['uuid']
            except Exception as e:
                log.error('An error occurred while extracting the agent/ip/uuid from {} to {}: {}'.format(agent_ip, url, e))
                return Response(status_code=400)

            # extract extra columns
            sysmon = data.get('systemmonitor', {})

            # response
            r = set_agent_monitoring_v1(agent, ip, uuid, sysmon, waf_id)

        elif url == del_agent_monitoring_v1_uri:

            # body request preprocessing
            r = await request_json_preprocess(url, request)
            if r[0]:
                data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
            else:
                return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

            # extract the data
            try:
                ids = data['id']
            except Exception as e:
                log.error('An error occurred while extracting the id from {} to {}: {}'.format(agent_ip, url, e))
                return Response(status_code=400)

            # response
            r = del_agent_monitoring_v1(ids, waf_id)

        else:

            # response
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# MLS
##

@app.post(get_mls_data_v1_uri)
@app.post(set_mls_data_v1_uri)
@app.post(del_mls_data_v1_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_mls_data_v1_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, _, waf_id, _ = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # data processing
        if url == get_mls_data_v1_uri:
            r = get_mls_data_v1(data, waf_id)
        elif url == set_mls_data_v1_uri:
            r = set_mls_data_v1(data, waf_id)
        elif url == del_mls_data_v1_uri:
            r = del_mls_data_v1(data, waf_id)
        else:
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)


##
# Antibot
##

@app.post(get_antibot_v1_uri)
@app.post(set_antibot_v1_uri)
@app.post(del_antibot_v1_uri)
@app.post(get_antibot_summary_v1_uri)
async def main(request: Request):

    # set URL
    url = request.scope['path']

    try:

        # init
        rsc = 200 if url in [get_antibot_v1_uri, get_antibot_summary_v1_uri] else 201

        # RW DB type only
        if ro_mode and (rsc == 201):
            return Response(status_code=502)

        # JSON request preprocessing
        r = await request_json_preprocess(url, request)
        if r[0]:
            data, key, waf_id, agent_ip = r[1][0], r[1][1], r[1][2], r[1][3]
        else:
            return JSONResponse(r[1], status_code=rsc) if isinstance(r[1], dict) else Response(status_code=r[1])

        # data processing
        if url == get_antibot_v1_uri:
            r = get_antibot_v1(waf_id)
        elif url == set_antibot_v1_uri:
            r = set_antibot_v1(data, waf_id)
        elif url == del_antibot_v1_uri:
            r = del_antibot_v1(data, waf_id)
        elif url == get_antibot_summary_v1_uri:
            geo_token_update(key, agent_ip)
            r = get_antibot_summary_v1(data, waf_id)
        else:
            log.error('An error occurred in {}: unknown URL'.format(url))
            return Response(status_code=400)

        # response
        return JSONResponse(r, status_code=rsc)

    except Exception as e:
        log.error('An error occurred in {}: {}'.format(url, e))
        return Response(status_code=503)
