--
-- PostgreSQL database dump
--

SET client_encoding = 'UTF8';
SET client_min_messages = warning;

--
-- Name: attack_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS attack_ids
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: attack; Type: TABLE
--

CREATE TABLE IF NOT EXISTS attack (
    id integer DEFAULT nextval('attack_ids'::regclass) NOT NULL,
    timestamp timestamp with time zone NOT NULL,
    lm boolean NOT NULL,
    vhost text NOT NULL,
    ip text NOT NULL,
    rule_id integer DEFAULT 0,
    method text NOT NULL,
    url text,
    param text,
    other_headers text,
    ua text,
    referer text,
    cookie text,
    body bytea,
    mz text,
    description text,
    long_description text,
    bt integer DEFAULT 0,
    request_id text,
    request_type text,
    waf_id text,
    group_id text,
    blocked boolean,
    possible boolean,
    cc char(2) DEFAULT NULL
);

CREATE UNIQUE INDEX IF NOT EXISTS attack_idx ON attack USING btree (id);
CREATE INDEX IF NOT EXISTS timestamp_attack_idx ON attack USING btree (timestamp);
CREATE INDEX IF NOT EXISTS ip_attack_idx ON attack USING btree (ip);

--
-- Name: scan_report_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS scan_report_ids
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: scan_report; Type: TABLE
--

CREATE TABLE IF NOT EXISTS scan_report (
    id integer DEFAULT nextval('scan_report_ids'::regclass) NOT NULL,
    scan_date timestamp without time zone NOT NULL,
    mz text,
    content text,
    domain text NOT NULL,
    method text NOT NULL,
    param text,
    payload text,
    type text,
    url text,
    data text,
    waf_id text NOT NULL
);

CREATE UNIQUE INDEX IF NOT EXISTS scan_report_idx ON scan_report USING btree (id);

--
-- Name: models; Type: TABLE
--

CREATE TABLE IF NOT EXISTS models (
    timestamp timestamp without time zone DEFAULT CURRENT_TIMESTAMP::timestamp without time zone,
    model bytea,
    waf_id text,
    vhost text,
    hash text
);

CREATE UNIQUE INDEX IF NOT EXISTS models_idx ON models USING btree (waf_id, vhost);

--
-- Name: training; Type: TABLE
--

CREATE TABLE IF NOT EXISTS training (
    waf_id text,
    vhost text,
    duration integer,
    complete boolean DEFAULT false,
    progress integer
);

CREATE UNIQUE INDEX IF NOT EXISTS training_idx ON training USING btree (waf_id, vhost);

--
-- Name: openapi_url_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS openapi_url_ids
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: openapi_url; Type: TABLE
--

CREATE TABLE IF NOT EXISTS openapi_url (
    id integer DEFAULT nextval('openapi_url_ids'::regclass) NOT NULL,
    timestamp timestamp with time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    waf_id text,
    url bytea
);

CREATE UNIQUE INDEX IF NOT EXISTS openapi_url_idx ON openapi_url USING btree (waf_id, md5(url));

--
-- Name: openapi_schemas_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS openapi_schemas_ids
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: openapi_schemas; Type: TABLE
--

CREATE TABLE IF NOT EXISTS openapi_schemas (
    id integer DEFAULT nextval('openapi_schemas_ids'::regclass) NOT NULL,
    timestamp timestamp with time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    waf_id text,
    name text,
    schema bytea
);

CREATE UNIQUE INDEX IF NOT EXISTS openapi_schemas_idx ON openapi_schemas USING btree (waf_id, name);

--
-- Name: openapi_schema_paths_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS openapi_schema_paths_ids
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: openapi_schema_paths; Type: TABLE
--

CREATE TABLE IF NOT EXISTS openapi_schema_paths (
    id integer DEFAULT nextval('openapi_schema_paths_ids'::regclass) NOT NULL,
    timestamp timestamp with time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    waf_id text,
    url text,
    schema bytea,
    schema_name text
);

CREATE UNIQUE INDEX IF NOT EXISTS openapi_schema_paths_idx ON openapi_schema_paths USING btree (waf_id, md5(url), schema_name);

--
-- Name: openapi_exclude_paths; Type: TABLE
--

CREATE TABLE IF NOT EXISTS openapi_exclude_paths (
    timestamp timestamp with time zone DEFAULT CURRENT_TIMESTAMP NOT NULL,
    waf_id text,
    exclude_paths bytea
);

CREATE UNIQUE INDEX IF NOT EXISTS openapi_exclude_paths_idx ON openapi_exclude_paths USING btree (waf_id);

--
-- Name: mt_main_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS mt_main_ids
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: mt_main; Type: TABLE
--

CREATE TABLE IF NOT EXISTS mt_main (
    id integer DEFAULT nextval('mt_main_ids'::regclass) NOT NULL,
    bt bigint,
    timestamp text,
    request_id text NOT NULL,
    mz text,
    mz_origin text,
    ip text,
    vhost text,
    method text,
    uri text,
    args text,
    cookie text,
    other_headers text,
    ua text,
    referer text,
    body text,
    waf_id text,
    agent text NOT NULL,
    cc char(2) DEFAULT NULL
);

CREATE UNIQUE INDEX IF NOT EXISTS mt_main_idx ON mt_main USING btree (id);
CREATE INDEX IF NOT EXISTS timestamp_mt_main_idx ON mt_main USING btree (timestamp);
CREATE INDEX IF NOT EXISTS ip_mt_main_idx ON mt_main USING btree (ip);

--
-- Name: mt_export_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS mt_export_ids
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: mt_export; Type: TABLE
--

CREATE TABLE IF NOT EXISTS mt_export (
    id integer DEFAULT nextval('mt_export_ids'::regclass) NOT NULL,
    bt bigint NOT NULL,
    timestamp text NOT NULL,
    request_id text NOT NULL,
    mz text NOT NULL,
    vhost text NOT NULL,
    method text NOT NULL,
    uri text NOT NULL,
    args text NOT NULL,
    cookie text NOT NULL,
    ua text NOT NULL,
    referer text NOT NULL,
    body text NOT NULL,
    other_headers text,
    waf_id text NOT NULL
);

CREATE UNIQUE INDEX IF NOT EXISTS mt_export_idx ON mt_export USING btree (id);

--
-- Name: dyn_sync_ban; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_sync_ban (
    waf_id text,
    key_id character varying(32)
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_sync_ban_idx ON dyn_sync_ban USING btree (key_id);

--
-- Name: dyn_bl; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_bl (
    active boolean DEFAULT true,
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    waf_id text,
    bl text
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_bl_idx ON dyn_bl USING btree (waf_id);

--
-- Name: dyn_wl_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS dyn_wl_ids
    AS integer
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: dyn_wl; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_wl (
    id integer DEFAULT nextval('dyn_wl_ids'::regclass),
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    active boolean DEFAULT true,
    waf_id text,
    domain text,
    mz text,
    extension text,
    rl_id text
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_wl_idx ON dyn_wl USING btree (id);

--
-- Name: dyn_wl_ts; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_wl_ts (
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    waf_id text
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_wl_ts_idx ON dyn_wl_ts USING btree (waf_id);

--
-- Name: dyn_erl_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS dyn_erl_ids
    AS integer
    START WITH 100000
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: dyn_erl; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_erl (
    id integer DEFAULT nextval('dyn_erl_ids'::regclass) NOT NULL,
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    active boolean DEFAULT true,
    waf_id text,
    ip text,
    lm boolean DEFAULT false,
    api boolean DEFAULT true,
    noban boolean DEFAULT false,
    country text,
    domain text,
    url text,
    url_x text,
    args text,
    args_x text,
    body text,
    body_x text,
    cookie text,
    cookie_x text,
    ua text,
    ua_x text,
    referer text,
    referer_x text,
    other_headers text,
    other_headers_x text,
    no_cookie boolean DEFAULT false,
    no_ua boolean DEFAULT false,
    no_referer boolean DEFAULT false,
    block_tor boolean DEFAULT false,
    block_proxy boolean DEFAULT false,
    zone_max_size text
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_erl_idx ON dyn_erl USING btree (id);

--
-- Name: dyn_erl_ts; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_erl_ts (
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    waf_id text
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_erl_ts_idx ON dyn_erl_ts USING btree (waf_id);

--
-- Name: dyn_settings; Type: TABLE
--

CREATE TABLE IF NOT EXISTS dyn_settings (
    active boolean DEFAULT true,
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    wl text,
    rl text,
    nwaf_ai_extra_host_lm text,
    nwaf_ai_extra_host_wl text,
    nwaf_bf_detect_host_lm text,
    nwaf_b64_decode_disable text,
    nwaf_body_exclude text,
    nwaf_body_bin_exclude text,
    nwaf_ddos_detect_host_lm text,
    nwaf_geoip_mla_disable text,
    nwaf_host_lm text,
    nwaf_host_wl text,
    nwaf_ip_lm text,
    nwaf_ip_wl text,
    nwaf_limit text,
    nwaf_mla_host_lm text,
    nwaf_put_body_exclude text,
    nwaf_rmq_host_exclude text,
    waf_id text,
    nwaf_url_wl text,
    nwaf_post_body_exclude text,
    nwaf_openapi_ip_wl text,
    nwaf_openapi_ip_lm text,
    nwaf_openapi_url_wl text,
    nwaf_openapi_url_lm text,
    nwaf_rfc_violation_wl text,
    nwaf_rfc_violation_lm text,
    nwaf_log_request text,
    nwaf_antibot_mem integer,
    nwaf_antibot_url text,
    nwaf_antibot_url_wl text,
    nwaf_antibot_ip_wl text,
    nwaf_antibot_cc_wl text,
    nwaf_av_ip_lm text,
    nwaf_av_ip_wl text,
    nwaf_av_url_lm text,
    nwaf_av_url_wl text
);

CREATE UNIQUE INDEX IF NOT EXISTS dyn_settings_idx ON dyn_settings USING btree (waf_id);

--
-- Name: mlc_settings; Type: TABLE
--

CREATE TABLE IF NOT EXISTS mlc_settings (
    active boolean DEFAULT true,
    modify timestamp without time zone DEFAULT (CURRENT_TIMESTAMP)::timestamp without time zone,
    main__ai_extra boolean,
    ddos__enable boolean,
    ddos__wl_ip text,
    ddos__wl_url text,
    ddos__interval integer,
    ddos__latest_only boolean,
    ddos__send_possible boolean,
    brute__enable boolean,
    brute__bf_detect text,
    brute__brute_detect text,
    brute__flood_detect text,
    brute__wl_ip text,
    brute__wl_host text,
    brute__interval integer,
    brute__max_val integer,
    brute__latest_only boolean,
    brute__send_possible boolean,
    waf_id text,
    openapi__vhosts_list text
);

CREATE UNIQUE INDEX IF NOT EXISTS mlc_settings_idx ON mlc_settings USING btree (waf_id);

--
-- Name: vhosts_list; Type: TABLE
--

CREATE TABLE IF NOT EXISTS vhosts_list (
    vhosts_list text,
    waf_id text
);

CREATE UNIQUE INDEX IF NOT EXISTS vhosts_list_idx ON vhosts_list USING btree (waf_id);

--
-- Name: nginx_global; Type: TABLE
--

CREATE TABLE IF NOT EXISTS nginx_global (
    active boolean DEFAULT true,
    waf_id text,
    resolver text,
    resolver_timeout text,
    set_real_ip_from text,
    real_ip_header text,
    client_max_body_size text,
    limit_req_zone text
);

CREATE UNIQUE INDEX IF NOT EXISTS nginx_global_idx ON nginx_global USING btree (waf_id);

--
-- Name: nginx_vhost; Type: TABLE
--

CREATE TABLE IF NOT EXISTS nginx_vhost (
    active boolean DEFAULT true,
    waf_id text NOT NULL,
    name text,
    listen text,
    server_name text,
    certificate_name text,
    dhparam_name text,
    ssl_session_timeout text,
    ssl_ciphers text,
    ssl_protocols text,
    root text,
    index text,
    add_header text,
    error_page text,
    locations text
);

CREATE UNIQUE INDEX IF NOT EXISTS nginx_vhost_idx ON nginx_vhost USING btree (waf_id, name);

--
-- Name: nginx_upstream; Type: TABLE
--

CREATE TABLE IF NOT EXISTS nginx_upstream (
    waf_id text NOT NULL,
    name text,
    balancing_method text,
    servers text
);

CREATE UNIQUE INDEX IF NOT EXISTS nginx_upstream_idx ON nginx_upstream USING btree (waf_id, name);

--
-- Name: nginx_certificate; Type: TABLE
--

CREATE TABLE IF NOT EXISTS nginx_certificate (
    waf_id text NOT NULL,
    name text,
    certificate text,
    certificate_key text
);

CREATE UNIQUE INDEX IF NOT EXISTS nginx_certificate_idx ON nginx_certificate USING btree (waf_id, name);

--
-- Name: nginx_dhparam; Type: TABLE
--

CREATE TABLE IF NOT EXISTS nginx_dhparam (
    waf_id text NOT NULL,
    name text,
    dhparam text
);

CREATE UNIQUE INDEX IF NOT EXISTS nginx_dhparam_idx ON nginx_dhparam USING btree (waf_id, name);

--
-- Name: events_ids; Type: SEQUENCE
--

CREATE SEQUENCE IF NOT EXISTS events_ids
    START WITH 1
    INCREMENT BY 1
    NO MINVALUE
    NO MAXVALUE
    CACHE 1;

--
-- Name: events; Type: TABLE
--

CREATE TABLE IF NOT EXISTS events (
    id integer DEFAULT nextval('events_ids'::regclass) NOT NULL,
    waf_id text,
    timestamp integer,
    ip text,
    uuid text,
    level text,
    agent text,
    category text,
    description bytea
);

CREATE UNIQUE INDEX IF NOT EXISTS events_idx ON events USING btree (id);

--
-- Name: geo_tokens; Type: TABLE
--

CREATE TABLE IF NOT EXISTS geo_tokens (
    waf_id text,
    geo_token text
);

CREATE UNIQUE INDEX IF NOT EXISTS geo_tokens_idx ON geo_tokens USING btree (waf_id, geo_token);

--
-- Name: nws_target; Type: TABLE
--

CREATE TABLE IF NOT EXISTS nws_target (
    active boolean DEFAULT true,
    waf_id text,
    target text,
    scan_proxy text,
    auth_params text,
    exclude_modules text
);

CREATE UNIQUE INDEX IF NOT EXISTS nws_target_idx ON nws_target USING btree (waf_id, target);

--
-- Name: agent_monitoring; Type: TABLE
--

CREATE TABLE IF NOT EXISTS agent_monitoring (
    id bigint PRIMARY KEY,
    waf_id text,
    agent text,
    ip text,
    uuid text,
    last_activity integer,
    systemMonitor text
);

CREATE UNIQUE INDEX IF NOT EXISTS agent_monitoring_idx ON agent_monitoring USING btree (waf_id, agent, uuid);

--
-- Name: mls_data; Type: TABLE
--

CREATE TABLE IF NOT EXISTS mls_data (
    id bigint PRIMARY KEY NOT NULL,
    timestamp integer,
    waf_id text NOT NULL,
    vhost text,
    type text,
    data text
);

--
-- Name: antibot_data; Type: TABLE
--

CREATE TABLE IF NOT EXISTS antibot_data (
    id bigint PRIMARY KEY NOT NULL,
    timestamp integer,
    waf_id text NOT NULL,
    ip text,
    ua character varying(32),
    fp character varying(32)
);

CREATE UNIQUE INDEX IF NOT EXISTS antibot_data_ip_ua_fp_idx ON antibot_data USING btree (ip, ua, fp);

--
-- Name: antibot_ua; Type: TABLE
--

CREATE TABLE IF NOT EXISTS antibot_ua (
    id character varying(32) PRIMARY KEY NOT NULL,
    timestamp integer,
    data text
);

--
-- Name: antibot_fp; Type: TABLE
--

CREATE TABLE IF NOT EXISTS antibot_fp (
    id character varying(32) PRIMARY KEY NOT NULL,
    timestamp integer,
    data text
);

--
-- PostgreSQL database dump complete
--
