#!/usr/bin/env python3

"""
Nemesida WAF API
Copyright (c) 2017-2025 Pentestit LLC - All Rights Reserved
Author: Romanov R.
"""

from time import time
from time import time_ns

from core import init_db_connection, close_db_connection
from logger import log


def get_mls_data_v1(data, waf_id):
    try:

        # init
        t = data.get('type')
        vhost = data.get('vhost')
        response = {'status': 'success', 'data': {}}

        # skip empty
        if not t:
            return {'status': 'fail', 'description': 'Empty or missing column: type'}

        # skip empty
        if (t not in ['openapi']) and (not vhost):
            return {'status': 'fail', 'description': 'Empty or missing column: vhost'}

        # init database connection
        connect = init_db_connection()
        cursor = connect.cursor()

        # init query
        if t in ['ml']:
            query = """ SELECT id, data FROM mls_data WHERE (waf_id = %s AND type = %s AND vhost = %s) """
            cursor.execute(query, (waf_id, t, vhost))
        else:
            query = """ SELECT id, data FROM mls_data WHERE (waf_id = %s AND type = %s) """
            cursor.execute(query, (waf_id, t,))

        # receive data from the database
        res = cursor.fetchall()
        close_db_connection(connect, cursor)

        # result processing
        for item in res:
            response['data'][item[0]] = item[1]

        # del vars
        del res

        # response
        return response

    except Exception as e:
        log.error('An error occurred in get_mls_data: {}'.format(e))
        return {'status': 'fail', 'description': str(e)}


def set_mls_data_v1(data, waf_id):
    try:

        # init
        rid = time_ns()
        ts = int(time())
        d = data.get('data')
        t = data.get('type')
        vhost = data.get('vhost')

        # skip empty
        if not t:
            return {'status': 'fail', 'description': 'Empty or missing column: type'}

        # skip empty
        if not d:
            return {'status': 'fail', 'description': 'Empty or missing column: data'}

        # skip empty
        if (t not in ['openapi']) and (not vhost):
            return {'status': 'fail', 'description': 'Empty or missing column: vhost'}

        # store data into the database
        query = """ INSERT INTO mls_data (id, timestamp, waf_id, vhost, type, data) VALUES (%s, %s, %s, %s, %s, %s) """
        connect = init_db_connection()
        cursor = connect.cursor()
        cursor.execute(query, (rid, ts, waf_id, vhost, t, d,))
        close_db_connection(connect, cursor)

        # response
        return {'status': 'success'}

    except Exception as e:
        log.error('An error occurred in set_mls_data: {}'.format(e))
        return {'status': 'fail', 'description': str(e)}


def del_mls_data_v1(data, waf_id):
    try:

        # init
        t = data.get('type')
        ids = data.get('ids')
        vhost = data.get('vhost')

        # skip empty
        if (not t) or (not ids) or (not vhost):
            return {'status': 'fail', 'description': 'Empty or missing column: type/ids'}

        # init database connection
        connect = init_db_connection()
        cursor = connect.cursor()

        # delete the data
        if '*' in ids:
            query = """ DELETE FROM mls_data WHERE (waf_id = %s AND type = %s AND vhost = %s) """
            cursor.execute(query, (waf_id, t, vhost))
        else:
            query = """ DELETE FROM mls_data WHERE (waf_id = %s AND type = %s AND vhost = %s AND id IN %s) """
            cursor.execute(query, (waf_id, t, vhost, tuple(ids),))

        # close database connection
        close_db_connection(connect, cursor)

        # processing result
        response = {'status': 'success'}

        # response
        return response

    except Exception as e:
        log.error('An error occurred in del_mls_data: {}'.format(e))
        return {'status': 'fail', 'description': str(e)}
