#!/usr/bin/python3

"""
Nemesida WAF API
Copyright (c) 2017-2025 Pentestit LLC - All Rights Reserved
Author: Romanov R.
"""

import logging
from time import time, sleep

from core import cc_upd_db
from core import ec_storage_period
from core import get_rldesc
from core import init_db_connection, close_db_connection
from core import ro_mode

##
# Log settings
##

logf = '/var/log/nw-api/rldscupd-logging.log'
log = logging.getLogger(__name__)
log.setLevel(logging.INFO)
formatter = logging.Formatter('%(asctime)s %(levelname)-8s %(message)s')
file_handler = logging.FileHandler(logf)
file_handler.setFormatter(formatter)
log.addHandler(file_handler)


def db_update():
    try:

        # RW DB type only
        if ro_mode:
            return

        # init database connection
        connect = init_db_connection()
        cursor = connect.cursor()

        ##
        # Processing rule description in attack
        ##

        sql = """ SELECT DISTINCT rule_id FROM attack WHERE description IN (NULL, '', '-') """
        cursor.execute(sql,)
        rows = cursor.fetchall()

        if len(rows):
            for i in rows:

                # init
                rule_id = str(i[0])

                if rule_id == '0':
                    continue

                # receive rule description if rule id not 0
                rldesc = get_rldesc(rule_id)
                log.info('Updated description for RuleID {}: {}'.format(rule_id, rldesc))

                # if rule description is not empty
                if rldesc:

                    # update description
                    query = """ UPDATE attack SET description = %s WHERE rule_id = %s """
                    cursor.execute(query, (rldesc, rule_id,))

        ##
        # Processing CC in attack
        ##

        for table in ['attack', 'mt_main']:
            cc_upd_db(table)

        ##
        # Delete old events
        ##

        try:

            # init
            eol_ts = int(time()) - (ec_storage_period * 24 * 3600)

            # delete the data
            sql = """ DELETE FROM events WHERE (timestamp < %s) """
            cursor.execute(sql, (eol_ts,))

        except Exception as e:
            log.error('An error occurred while deleting old events: {}'.format(e))

        ##
        # Delete old mls data
        ##

        try:

            # init
            eol_ts = int(time()) - (3 * 24 * 3600)  # 3 days

            # delete the data
            sql = """ DELETE FROM mls_data WHERE (timestamp < %s) """
            cursor.execute(sql, (eol_ts,))

        except Exception as e:
            log.error('An error occurred while deleting old MLS data: {}'.format(e))

        ##
        # Delete old antibot data
        ##

        try:

            # init
            eol_ts = int(time()) - (31 * 24 * 3600)  # 1 month

            # delete the data
            sql = """ DELETE FROM antibot_data WHERE (timestamp < %s) """
            cursor.execute(sql, (eol_ts,))

        except Exception as e:
            log.error('An error occurred while deleting old antibot data: {}'.format(e))

        # close database connection
        close_db_connection(connect, cursor)

    except Exception as e:
        log.error('An error occurred while processing data: {}'.format(e))


if __name__ == '__main__':
    while True:
        db_update()
        sleep(60)
